# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2009
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    Classe de gestion des users 'Eleves'
"""
from glob import glob
from os import system, makedirs, unlink
from os.path import join, isdir
from ldap import MOD_REPLACE
from scribe.ldapconf import USER_DN, SMB_SERVEUR, \
HOME_PATH, MAIL_DOMAIN, CLASS_PATH, BRANCHE, ELEVE_FILTER, OPT_PATH
from scribe.eoletools import set_default_acl, add_dir, set_acl, \
add_symlink, format_old_date, replace_cars, fixdate, not_empty, \
format_current_date, deformate_date
from scribe.eoleuser import User, gen_common_attrs, send_first_mail, \
gen_enteleve_attrs, gen_profil, gen_common_devdir, gen_radius_attrs


class Eleve(User):
    """
        classe pour la gestion des élèves Scribe
    """
    _type = 'eleves'
    profil = 'eleve'
    filtre = "(&%s)" % ELEVE_FILTER
    must_args = ['login', 'password', 'nom', 'prenom', 'date',
    'classe', 'numero', 'civilite']
    may_args = dict(domaine='restreint',
                    quota='10',
                    profil='1',
                    shell=False,
                    change_pwd=False,
                    ine='',
                    patronyme='',
                    prenom2='',
                    int_id='',
                    entlogin=True,
                    regime='',
                    rattachid='',
                    )
    has_samba = True
    has_ftp = True
    user_add_params = """-a -A 1 -D "U:" -H U -o "ou=local,ou=eleves" \
-s /bin/%%(shell)s \
-d "/home/%%(login_first)s/%%(login)s" \
-C '\\\\%s\\%%(login)s\\perso' \
-F '%%(profil)s' -g "eleves" \
-G "%%(classe)s,%%(niveau)s,DomainUsers" \
"%%(login)s" """ % SMB_SERVEUR

    def get_smbldap_useradd_args(self, login, profil, shell, classe, **args):
        """
            ajoute un utilisateur à l'aide /usr/sbin/smbldap-useradd
        """
        if shell:
            shell = "bash"
        else:
            shell = "false"
        arg_dict = {"shell":shell,
                    "login_first":login[0],
                    "login":login,
                    "profil":gen_profil(profil, login),
                    "classe":classe,
                    "niveau":self._get_niveau(classe)
                    }
        return arg_dict

    def _add_scribe_user(self, login, **args):
        """
            ajout les attributs Scribe à un Eleve
        """
        mail = "%s@%s" % (login, MAIL_DOMAIN[args['domaine']])
        maildir = join(HOME_PATH, login[0], login, 'MailDir') + '/'
        args['niveau'] = self._get_niveau(args['classe'])
        datas = []
        user_dn = USER_DN % dict(uid=login, _type=self._type)
        objectclass = self._get_attr(login, 'objectClass')
        objectclass.extend(['Eleves', 'ENTPerson', 'ENTEleve', 'radiusprofile'])
        datas.append((MOD_REPLACE, 'objectClass', objectclass))
        datas.extend(gen_common_attrs(login, entprofil=self.profil, **args))
        datas.append((MOD_REPLACE, 'employeeNumber', args['numero']))
        datas.append((MOD_REPLACE, 'Ine', args['ine'] ))
        datas.append((MOD_REPLACE, 'mail', mail))
        datas.append((MOD_REPLACE, 'mailHost', 'localhost'))
        datas.append((MOD_REPLACE, 'mailDir', maildir))
        datas.append((MOD_REPLACE, 'Divcod', args['classe']))
        datas.append((MOD_REPLACE, 'Meflcf', args['niveau']))
        #push(@mods,'rne'=>"$rne");
        datas.extend(gen_enteleve_attrs(**args))
        datas.extend(gen_radius_attrs())
        self.ldap_admin._modify(user_dn, datas)
        send_first_mail(mail)

    def _add_perso(self, login, **args):
        """
            Crée les différents répertoires de l'utilisateur
        """
        # remplace /usr/share/ead/rep-eleve.sh
        abc = join(HOME_PATH, login[0])
        rep = join(abc, login)
        perso = join(rep, 'perso')
        maildir = join(rep, 'MailDir')
        cl_dir = join(CLASS_PATH, args['classe'])
        if not isdir(abc):
            makedirs(abc)
        system("setfacl -m u:mail:r-x,g:mail:r-x %s" % (abc,))
        # TODO : répertoire privé optionnel ?
        add_dir(join(perso, 'prive'))
        add_dir(maildir)
        system("setfacl -PRm u:%s:rwx,g::---,o::--- %s" % (login, rep))
        # mail
        system("setfacl -m u:mail:r-x,g:mail:r-x %s" % (rep,))
        system("setfacl -m u:mail:rwx,g:mail:rwx %s" % (maildir,))
        # accès profs
        set_acl(rep, 'professeurs', 'r-x', recursive=False)
        set_acl(perso, 'professeurs', 'r-x', recursive=False)
        # acl par défaut
        set_default_acl(rep, recursive=False)
        set_default_acl(perso, recursive=False)
        set_default_acl(join(perso, 'prive'), recursive=False)
        set_default_acl(maildir, recursive=False)
        # pour les quotas
        system('chown -R %s %s' % (login, perso))
        # pour courier-imap (hardy)
        system('chown mail:mail %s' % (maildir,))
        # lien direct depuis profs-classe
        add_dir(cl_dir)
        add_symlink(perso, join(cl_dir, login))

    def _gen_devdir(self, login, homedir=None):
        """
        Génération du répertoire "devoirs" pour les élèves
        """
        if homedir is None:
            homedir = self._get_attr(login, 'homeDirectory')[0].strip()
        gen_common_devdir(login, homedir)
        # pas de partie spécifique pour les élèves

    def _change_classe(self, user, new_classe):
        """
        Changement de classe
        @param user : login de l'élève
        @param new_classe : nouvelle classe de l'élève
        """
        old_classe = self._get_attr(user, 'Divcod')[0]
        old_niveau = self._get_attr(user, 'Meflcf')[0]
        new_niveau = self._get_niveau(new_classe)
        # modification utilisateur
        self._set_attr(user, 'Divcod', new_classe)
        self._set_attr(user, 'Meflcf', new_niveau)
        # attributs ENT (à revoir ?)
        self._set_attr(user, 'ENTEleveClasses', "%s$%s" % (BRANCHE, new_classe))
        self._set_attr(user, 'ENTEleveMEF', new_niveau)
        self._set_attr(user, 'ENTEleveLibelleMEF', new_niveau)
        self._touch(user)
        # désinscriptions
        self._desinscription(user, old_classe, sync=False)
        self._desinscription(user, old_niveau, sync=False)
        # inscriptions
        self._inscription(user, new_classe, sync=False)
        self._inscription(user, new_niveau, sync=False)
        # mise à jour du ".ftp"
        self._gen_ftpdir(user)
        return True

    def _inscription(self, login, groupe, sync=True):
        """
        réécriture de l'inscription
        pour prendre en compte les liens "Option"
        """
        # appel à la classe mère
        User._inscription(self, login, groupe, sync=sync)
        # gestion des cas Option et Classe
        opt_dir = join(OPT_PATH, groupe)
        cl_dir = join(CLASS_PATH, groupe)
        for directory in [opt_dir, cl_dir]:
            if isdir(directory):
                perso = join(HOME_PATH, login[0], login, 'perso')
                add_symlink(perso, join(directory, login))

    def _desinscription(self, login, groupe, sync=True):
        """
        réécriture de la désinscription
        pour prendre en compte les liens "Option"
        """
        # appel à la classe mère
        User._desinscription(self, login, groupe, sync=sync)
        # gestion des cas Option et Classe
        user_opt_dir = join(OPT_PATH, groupe, login)
        user_cl_dir = join(CLASS_PATH, groupe, login)
        for directory in [user_opt_dir, user_cl_dir]:
            if isdir(directory):
                unlink(directory)

    def _update(self, login, **args):
        """
            Mise à niveau Eleve (import)
        """
        user_dn = USER_DN % dict(uid=login, _type=self._type)
        datas = []
        if not_empty(args, 'ine'):
            datas.append((MOD_REPLACE, 'Ine', args['ine']))
        if not_empty(args, 'int_id'):
            datas.append((MOD_REPLACE, 'intid', args['int_id']))
        if not_empty(args, 'rattachid'):
            datas.append((MOD_REPLACE, 'ENTEleveStructRattachId', args['rattachid']))
        if not_empty(args, 'regime'):
            # FIXME : codes Sconet !
            datas.append((MOD_REPLACE, 'ENTEleveRegime', args['regime']))
        datas.append((MOD_REPLACE, 'sn', args['nom']))
        datas.append((MOD_REPLACE, 'givenName', args['prenom']))
        date = deformate_date(args['date'])
        datas.append((MOD_REPLACE, 'dateNaissance', date))
        datas.append((MOD_REPLACE, 'ENTPersonDateNaissance', date))
        datas.append((MOD_REPLACE, 'LastUpdate', format_current_date()))
        self.ldap_admin._modify(user_dn, datas)

    def _Upgrade(self, login):
        """
        Mise à niveau d'un compte élève
        """
        user_dn = USER_DN % dict(uid=login, _type=self._type)
        datas = []
        obj = self._get_attr(login, 'objectClass')
        obj.extend(['ENTPerson', 'ENTEleve'])
        datas.append((MOD_REPLACE, 'objectClass', obj))
        datas.extend(self._gen_upgrade_attrs(login))
        if 'radiusprofile' not in obj:
            obj.append('radiusprofile')
            datas.extend(gen_radius_attrs())
        self.ldap_admin._modify(user_dn, datas)

    def _gen_upgrade_attrs(self, login):
        """
        Gestion des nouveaux attributs élève
        """
        infos = self._get_attrs(login, ['givenName', 'sn', 'Divcod',
                          'codecivilite', 'dateNaissance', 'Meflcf'])
        prenom = infos['givenName'][0]
        nom = infos['sn'][0]
        classe = infos['Divcod'][0]
        civilite = infos['codecivilite'][0]
        niveau = infos['Meflcf'][0]
        date = infos['dateNaissance'][0]
        attrs = []
        yesterday = format_old_date()
        attrs.append((MOD_REPLACE, 'LastUpdate', yesterday))
        attrs.append((MOD_REPLACE, 'gecos', replace_cars("%s %s" % (prenom, nom)) ))
        attrs.append((MOD_REPLACE, 'ENTPersonLogin', login))
        attrs.append((MOD_REPLACE, 'ENTPersonJointure', 'ENT'))
        attrs.append((MOD_REPLACE, 'ENTPersonProfils', self.profil))
        attrs.append((MOD_REPLACE, 'ENTPersonNomPatro', nom))
        if civilite == '3':
            sexe = 'F'
            titre = 'Mlle'
        elif civilite == '2':
            sexe = 'F'
            titre = 'Mme'
        else:
            titre = 'M.'
            sexe = 'M'
        attrs.append((MOD_REPLACE, 'ENTPersonSexe', sexe))
        attrs.append((MOD_REPLACE, 'personalTitle', titre))
        attrs.append((MOD_REPLACE, 'ENTPersonDateNaissance', fixdate(date)))
        attrs.extend(gen_enteleve_attrs(niveau=niveau, classe=classe, rattachid=''))
        return attrs

    def _get_eleves(self, attrs=['uid']):
        """
            renvoie les élèves
        """
        return self._get_users(filtre=self.filtre, attrs=attrs)

    def is_eleve(self, uid):
        """
            test si l'utilisateur existe dans l'annuaire
            (mode déconnecté)
        """
        self.ldap_admin.connect()
        res = self._is_eleve(uid)
        self.ldap_admin.close()
        return res

    def _is_eleve(self, uid):
        """
            test si l'utilisateur existe dans l'annuaire
        """
        uidfilter = "(&%s(uid=%s))" % (ELEVE_FILTER, uid)
        if self.ldap_admin._search_one(uidfilter):
            return True
        return False

    def _delete(self, login, remove_data=False, need_connect=False, delete_resp=False):
        """
            surcharge de la méthode _delete
            pour gérer les associations élève-responsable
        """
        from scribe.responsables import Responsable
        resp = Responsable()
        if need_connect:
            resp.ldap_admin.connect()
        else:
            resp.ldap_admin = self.ldap_admin
        responsables = resp._get_responsables(['uid', 'eleve'], login)
        if delete_resp:
            # suppression si nécessaire
            for responsable in responsables:
                if len(responsable['eleve']) == 1:
                    resp._delete(responsable['uid'][0], remove_data=remove_data, need_connect=False)
                else:
                    resp.c_del_association(responsable['uid'][0], login)
        else:
            # pas de suppression -> vérification préalable
            for responsable in responsables:
                if len(responsable['eleve']) == 1:
                    raise Exception("Suppression impossible : %s est le dernier élève du responsable %s" % (login, responsable['uid'][0]))
            for responsable in responsables:
                resp.c_del_association(responsable['uid'][0], login)
        if need_connect:
            resp.ldap_admin.close()
        # ménage dans les liens symboliques
        eleve_links = glob('%s/*/%s' % (CLASS_PATH, login))
        eleve_links.extend(glob('%s/*/%s' % (OPT_PATH, login)))
        for link in eleve_links:
            unlink(link)
        # appel à la classe mère
        User._delete(self, login, remove_data, need_connect)

